unit Preview1;
// This is the basic main form for this demo app.
// If shows two buttons and a selection
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DB, DBTables, Mask, DBCtrls, ShellAPI, PrintDrv, Printers,
  ExtCtrls, Buttons, OLEAuto, FileCtrl;

const
	LeftMargin:double = 0.5;
  RightMargin:double = 8.0;
  TopMargin:double = 0.5;
  LetterBottomMargin:double = 10.5;
  LegalBottomMargin:double = 13.5;

  olMailitem = 0 ;
  olImportanceLow = 0 ;
  olImportanceNormal = 1 ;
  olImportanceHigh = 2 ;


type
  TMainForm = class(TForm)
    Label1: TLabel;
    Demo1: TButton;
    PrintOrient: TCheckBox;
    Image1: TImage;
    Demo2: TButton;
    MemoDemo: TMemo;
    Prn: TPrintObject;
    LegalSize: TCheckBox;
    SpeedButton1: TSpeedButton;
    procedure Demo1Click(Sender: TObject);
    procedure Demo2Click(Sender: TObject);
    procedure PrnNewPage(Sender: TObject);
    procedure SetPageDefaults;
    procedure PrintOrientClick(Sender: TObject);
    procedure LegalSizeClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { Private declarations }
    procedure DoTestPreview;
    procedure PrintTestFile;
    procedure PrintTestDemo;
    procedure CreateColumns;
    procedure TestPrintTest;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  TestNumber: integer;
  Shading: integer;
  BottomMargin: double;
  CancelMe: boolean;

implementation

{$R *.DFM}

procedure TMainForm.SetPageDefaults;
// I usually create separate default routines so that I can call them separately
// based on the type of report I want to perform.
begin
	with Prn do
  begin
    	SetFontInformation('Times New Roman',10,[]);
      if PaperSize = Letter then
      	BottomMargin := LetterBottomMargin
      else
      	BottomMargin := LegalBottomMargin;
      SetMargins(TopMargin,BottomMargin,LeftMargin,RightMargin);
    	SetHeaderInformation(1,0.625,'Test Preview Program',1,'Times New Roman',16,[fsBold]);
    	SetHeaderInformation(2,0.850,'Second Header Line',1,'Times New Roman',12,[]);
      SaveCurrentFont;
    	SetFontInformation('Times New Roman',16,[fsBold]);
    	SetHeaderDimensions(0.25,0.25,RightMargin,0.625 + (GetLineHeightInches * 2),False,0,255);
      SetDetailTopBottom(0.625 + (GetLineHeightInches * 2),BottomMargin-0.5);
      RestoreCurrentFont;
      case TestNumber of
      	1: SetPageNumberInformation(0.5,'Page %d',2,True,'Times New Roman',10,[]);
        2: SetPageNumberInformation(BottomMargin + GetLineHeightInches,'Page %d',1,
        		True,'Times New Roman',10,[]);
      end;
    	SetFooterDimensions(LeftMargin,BottomMargin-0.5,RightMargin,BottomMargin,False,0,255);
      SetFooterInformation(1,BottomMargin,'Page Footer Line #1.',1,'Arial',10,[]);
      MemoDemo.Font.Name := 'TimesNewRoman';
      MemoDemo.Font.Size := 10;
      MemoDemo.Font.Style := [];
  end;
end;

Procedure TMainForm.DoTestPreview;
// entry point for both demo print routines to set up the defaults
var
	TempX: single;

begin
  try
//  	Hide;
    Shading := 235;
    CancelMe := false;
    with Prn do
    begin
      if PrintOrient.Checked then
      begin
      	Orientation := poLandscape;
    		RightMargin := 10.5;
        BottomMargin := 8.0;
      end
      else
      begin
      	Orientation := poPortrait;
    		RightMargin := 8.0;
        BottomMargin := 10.5;
      end;
  		Start;
      SetPageDefaults;

			MemoDemo.Width := InchesToPixelsHorizontal(3.2);
  		MemoDemo.Text := '     This is some text put into a memo box. In order to '+
  										 'print the text of a memo control, I usually adjust the '+
          		         'size of the memo box to the width of the text as I want '+
              		     'it to appear on the form. This makes use of the automatic '+
                  		 'word wrap feature. Then just loop through the lines and '+
		                   'print! This text should wrap inside of a 3.5" box.';
      case TestNumber of
      	1: PrintTestFile;
        2: PrintTestDemo;
      end;
      NextLine;
      WriteLineCenter(-1,'*** End of test ***');
      if Not CancelMe then
      	Quit
      else
      	Abort;
    end;
  finally
  	Show;
  end;
end;

procedure TMainForm.PrintTestFile;
// demo printing for AutoPaging set to true
label
	DoneProcessing;
var

	tFile: TextFile;
  sLine: string;
  fY: integer;
  fX,fW: single;
  EveryOther: boolean;

begin
	try
		with Prn do
  	begin
    	if Not FileExists(ExtractFilePath(ParamStr(0))+'Readme.Txt') then
      begin
      	ShowMessage('Could not find test file...');
        exit;
      end;
      AutomaticPaging := true;
			AssignFile(tFile,ExtractFilePath(ParamStr(0))+'Readme.Txt');
      Reset(tFile);
      fY := 0;
      SetYPosition(PixelsToInchesVertical(Image1.Height div 2) + TopMargin);
      while not Eof(tFile) do //first loop to get a test count for progress bar...
      begin
      	readln(tFile,sLine);
    		if Not PrintingStatus then goto DoneProcessing;
        inc(fY);
      end;
      SetProgressMax(fY);
      CloseFile(tFile);
			AssignFile(tFile,ExtractFilePath(ParamStr(0))+'Readme.Txt');
      Reset(tFile);
      CreateColumn(1,LeftMargin,RightMargin-LeftMargin);
      SetTopOfPage;
      NextLine;
      EveryOther := false;
      while not Eof(tFile) do
      begin
      	Readln(tFile,sLine);
        if EveryOther then
        begin
        	NextLine;
        	WriteLine(LeftMargin, -2.0,sLine);
        end
        else
        	WriteLine(LeftMargin,-1,sLine);
        EveryOther := not EveryOther;
        if trim(sLine) <> '' then
        	DrawLine(LeftMargin,GetYPosition + GetLineHeightInches,RightMargin,
        				GetYPosition + GetLineHeightInches,1);
    		if Not PrintingStatus then goto DoneProcessing;
        Increment;
      end;
DoneProcessing:
			CloseFile(tFile);
  	end;
  except
  	on Exception do
    	ShowMessage('Sorry, something didn''t work right!');
  end;
end;

procedure TMainForm.PrintTestDemo;
// demo printing for AutoPaging set to false for manual control.
var
	x,y: integer;
  z: single;
  s: string;

begin
	with Prn do
  begin
  	ProgressBarInc := 2;
  	AutomaticPaging := false;
		SetTopOfPage;
    CreateColumns;
    DrawTextBox(GetColumnLeft(1),GetYPosition - 0.02,GetColumnRight(1),
    						GetYPosition + GetLineHeightInches + 0.02,1,Shading,
                'Column 1',1,1);
    DrawTextBox(GetColumnLeft(2),GetYPosition - 0.02,GetColumnRight(2),
    						GetYPosition + GetLineHeightInches + 0.02,1,Shading,
                'Column 2',2,1);
    DrawTextBox(GetColumnLeft(3),GetYPosition - 0.02,GetColumnRight(3),
    						GetYPosition + GetLineHeightInches + 0.02,1,Shading,
                'Column 3',3,1);
    DrawTextBox(GetColumnLeft(4),GetYPosition - 0.02,GetColumnRight(4),
    						GetYPosition + GetLineHeightInches + 0.02,1,Shading,
                'Column 4',4,1);
		NextLine;
    IncBar(10);
    WriteLine(LeftMargin,-1.0,'Centered column text...');
    NextLine;
    for x := 1 to 10 do
    begin
    	for y := 1 to 4 do
      	WriteLineColumnCenter(y,GetYPosition,'Line: ' + IntToStr(x) + '  Col: ' + IntToStr(y));
      NextLine;
      Increment;
    end;
    if CancelMe then exit;
    NextLine;
    WriteLine(LeftMargin,-2.0,'Left justified column text...');
    NextLine;
    for x := 1 to 10 do
    begin
    	for y := 1 to 4 do
      	WriteLineColumn(y,-2.0,'Line: ' + IntToStr(x) + '  Col: ' + IntToStr(y));
      NextLine;
      Increment;
    end;
    if not PrintingStatus then exit;
    NextLine;
    WriteLine(LeftMargin,-1.0,'Right justified column text...');
    NextLine;
    for x := 1 to 10 do
    begin
    	for y := 1 to 4 do
      	WriteLineColumnRight(y,-2.0,'Line: ' + IntToStr(x) + '  Col: ' + IntToStr(y));
      NextLine;
      Increment;
    end;
    Newlines(3);
    PrintGraphic(LeftMargin,BottomMargin-0.2,PixelsToInchesHorizontal(Image1.Width),
    							PixelsToInchesVertical(Image1.Height),Image1);
    {
    	OR using -1.0 to default to graphic width and height.....
    PrintGraphic(LeftMargin,GetYPosition,-1.0,-1.0,Image1);
    }
    if not PrintingStatus then exit;
    NewPage;
    SetTab(0.1);
    WriteLine(LeftMargin,-1.0,'[Shaded Box]');
    SetTab(0.0);
    Increment;
    DrawBoxShaded(LeftMargin,GetYPosition - 0.1,1.5,3.0,4,Shading);
    Increment;
    if CancelMe then exit;
    WriteLine(2.6,-2.0,'[Regular Box]');
    DrawBox(2.4,GetYPosition - 0.1,4.0,GetYPosition + 0.3,1);
    Increment;
    SetYPosition(3.0);
    WriteLineCenter(-1.0,'Center Justification');
    Increment;
    if CancelMe then exit;
    SaveCurrentFont;
    SetFontInformation('Arial',8,[fsBold,fsUnderline]);
    WriteLineRight(-2.0,'Right Justified Text [fsBold,fsUnderline]');
    RestoreCurrentFont;
    Increment;
    if CancelMe then exit;
    SetYPosition(6.0);
    WriteLine(LeftMargin,-1.0,'Text printed without a tab set...');
    SetTab(0.25);
    WriteLine(LeftMargin,-1.0,'Text printed using a 0.25" tab setting...');
    SetTab(0.0);
    NextLine;
    WriteLine(LeftMargin,-1.0,'Equation:  2x');
    x := GetTextWidth('Equation:  2x ');
    if CancelMe then exit;
    SaveCurrentFont;
    SetFontInformation(GetFontName,6,[]);
    WriteLine(LeftMargin + PixelsToInchesHorizontal(x),GetYPosition - 0.02,'2');
    x := x + GetTextWidth('2');
    RestoreCurrentFont;
    WriteLine(LeftMargin + PixelsToInchesHorizontal(x),-2.0,' = 12');
    NextLine;
    if CancelMe then exit;
    WriteLine(LeftMargin,-1.0,'Memo field printing...');
    DrawBox(LeftMargin,GetYPosition + GetLineHeightInches,LeftMargin + 3.8,
    				GetYPosition + ((MemoDemo.Lines.Count + 1) * GetLineHeightInches) + 0.05,1);
    SetTab(0.05);
  // Command to wrap text within a given bounds...
    WriteLineWrap(LeftMargin,3.5,-1.0,MemoDemo.Text,true,true);
  // Same command but wrapping text within a column:
    WriteLineColumnWrap(3,-2.0,MemoDemo.Text,false,true);
    NewLines(2);
    s := 'This is a long line of text to ensure that the word wrap of the columns ' +
    			'are taking place. Since this is starting close to the bottom of the 2nd ' +
          'page, it should start a new page before printing the columns.';
    WriteLineColumnArray(GetYPosition,[1,2,3,4],[1,0,0,1],['col 1',s,'col 3','col 4'],false,true);
    NextLine;
    z := GetYPosition;
    WriteLineWrap(LeftMargin,3.5,-1.0,MemoDemo.Text,true,false);
    WriteLineColumn(3,-2.0,'This is just a test line...');
    NextLine;
    UpdateBar(100);
  end;
end;

procedure TMainForm.TestPrintTest;
var
	s: string;
  oldPos: real;
  x: integer;

begin
 	Prn.Preview := true;
	with Prn do
  begin
  	Start;
    ProgressBarInc := 10;

    CreateColumn(1,LeftMargin,1.0);
    CreateColumn(2,GetColumnRight(1) + 0.1,0.8);
		CreateColumn(3,GetColumnRight(2) + 0.3,1.0);
    CreateColumn(4,GetColumnRight(3) + 0.1,0.8);
    CreateColumn(5,GetColumnRight(4) + 0.3,1.0);
    CreateColumn(6,GetColumnRight(5) + 0.1,0.8);
    CreateColumn(10,LeftMargin,1.4);
    CreateColumn(7,GetColumnRight(10) + 0.1,1.0);
    CreateColumn(8,GetColumnRight(7) + 0.1,1.0);
    CreateColumn(9,GetColumnRight(8) + 0.1,1.0);
		CreateColumn(11,GetColumnRight(4),2.8);

		BottomMargin := LetterBottomMargin;
    SetMargins(TopMargin,BottomMargin,LeftMargin,RightMargin);
    SetFontInformation('Times New Roman',10,[]);
    SetDetailTopBottom(TopMargin + 1.0,BottomMargin - 1.0);
    SetHeaderDimensions(LeftMargin,TopMargin,RightMargin,TopMargin + 0.5, false, 1, 255);

		SetPageDefaults;
    SetTopOfPage;

    s := 'This is a long line of text to ensure that the word wrap of the columns ' +
    			'are taking place. Since this is starting close to the bottom of the 2nd ' +
          'page, it should start a new page before printing the columns.';


    NewLines(25);
		SaveCurrentY;
    	SetFontInformation(GetFontName,GetFontSize,[fsBold]);
    	WriteLineColumn(1,-2.0,'Order Date');
    	WriteLineColumn(4,-2.0,'Comment');
    	SetFontInformation(GetFontName,GetFontSize,[]);
      NextLine;
      for x := 1 to 15 do
      	WriteLine(LeftMargin,-1.0,'testing');
			RestoreCurrentY;
      NextLine;
      SetFontInformation(GetFontName,GetFontSize,[fsBold]);
      WriteLineColumn(1,-2.0,'Due Date');
      SetFontInformation(GetFontName,GetFontSize,[]);
  	Quit;
  end;
end;

procedure TMainForm.CreateColumns;
{This procedure sets up a few columns}
begin
	with Prn do
  begin
  	CreateColumn(1,0.5,1.5); //column 1
    CreateColumn(2,2.5,1.5); //column 2
    CreateColumn(3,4.5,1.5); //column 3
    CreateColumn(4,6.5,1.5); //column 4
  end;
end;

procedure TMainForm.Demo1Click(Sender: TObject);
// do automatic paging demo
begin
	TestNumber := 1;
	DoTestPreview;
end;

procedure TMainForm.Demo2Click(Sender: TObject);
// do manual paging demo
begin
	TestNumber := 2;
	DoTestPreview;
end;

procedure TMainForm.PrnNewPage(Sender: TObject);
// This routine is called in the OnNewPage event to place my graphic
// at the top of each page.
begin
	Prn.PrintGraphic(0.25,0.25,-1.0,-1.0,Image1);
end;

procedure TMainForm.PrintOrientClick(Sender: TObject);
begin
	if TCheckBox(Sender).Checked then
  	Prn.Orientation := poLandscape
  else
  	Prn.Orientation := poPortrait;
end;

procedure TMainForm.LegalSizeClick(Sender: TObject);
begin
	if TCheckBox(Sender).Checked then
  	Prn.PaperSize := Legal
  else
  	Prn.PaperSize := Letter;
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
	TestPrintTest;
end;

end.
